import { Component, Input, OnInit } from '@angular/core';
import { FormBuilder, UntypedFormGroup, Validators } from '@angular/forms';
import { NbDialogRef } from '@nebular/theme';
import { User } from 'app/core/models/user';
import { environment } from 'environments/environment';
import { StaffGroup } from '../../models/staff-group';

@Component({
  selector: 'ngx-dialog-add-staff-membership',
  templateUrl: './dialog-add-staff-membership.component.html',
  styleUrls: ['./dialog-add-staff-membership.component.scss']
})
export class DialogAddStaffMembershipComponent implements OnInit {
  membershipForm: UntypedFormGroup;
  StaffGroup = StaffGroup;
  user: User;
  permissionManualUrl = environment.permissionManualUrl;

  get staffGroups(): StaffGroup[] {
    return <any>Object.keys(StaffGroup).filter(k => !isNaN(Number(k))).map(k => Number(k));
  }

  constructor(
    protected ref: NbDialogRef<DialogAddStaffMembershipComponent>,
    private fb: FormBuilder) {
  }

  createForm() {
    this.membershipForm = this.fb.group({
      user: [null, [Validators.required]],
      group: [null, [Validators.required]]
    });
  }

  ngOnInit(): void {
    this.createForm();
  }

  onUserSelected(user: User) {
    this.user = user;
    this.membershipForm.patchValue({
      user: user
    });
  }

  cancel() {
    this.ref.close();
  }

  add() {
    console.log(this.membershipForm.value);
    this.ref.close({
      user: this.membershipForm.value.user,
      group: this.membershipForm.value.group
    });
  }
}
