import { HttpClient, HttpParams } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { BaseService } from "app/shared/services/base.service";
import { environment } from "environments/environment";
import { Observable } from "rxjs";
import { catchError, map } from "rxjs/operators";
import { Course } from "../models/course";
import { HttpCacheManager, withCache } from '@ngneat/cashew';
import { AppStateService } from "app/core/services/app.state.service";
import { UpdateCourseDto } from "../dto/update-course.dto";
import { BackendError } from "app/shared/models/backend-error";
import { CreateCourseDto } from "../dto/create-course.dto";
import { CourseStaffMembership } from "../models/course-staff-membership";
import { StaffGroup } from "../models/staff-group";
import { CourseUnit } from "../models/course-unit";
import { CourseFile } from "../models/course-file";
import { CreateCourseFileDto } from "../dto/create-course-file.dto";
import { DownloadService } from "app/shared/services/download.service";

@Injectable({
    providedIn: 'root'
})
export class CourseFileService extends BaseService {
    constructor(
        private http: HttpClient, 
        private appStateService: AppStateService,
        private downloadService: DownloadService, 
        private cacheManager: HttpCacheManager
        ) {
        super();
    }

    getById(id: number): Observable<Course> {
        return this.http.get<Course>(`${environment.backendUrl}/course/${id}`, {
            context: withCache() 
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(course => new Course(course))
        )
    }

    getFileUrl(file: CourseFile): string {
        return `${environment.backendUrl}/course_file/${file.id}/download`;
    }

    downloadFile(file: CourseFile): Observable<void> {
        return this.downloadService.downloadFile(this.getFileUrl(file), true);
    }

    create(dto: CreateCourseFileDto, file: File): Observable<CourseFile> {
        var formData = new FormData();
        if(file) {
            formData.append("file", file, file.name);
        }
        
        formData.append("metadata", new Blob([JSON.stringify(dto)], { type: 'application/json' }));

        return this.http.post<CourseFile>(`${environment.backendUrl}/course_file`, formData).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(response => new CourseFile(response))
        );
    }

    delete(id: number): Observable<void> {
        return this.http.delete<void>(`${environment.backendUrl}/course_file/${id}`).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(() => this.cacheManager.clear())
        );
    }
}
