import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { environment } from 'environments/environment';
import { Observable, of } from 'rxjs';
import { Term } from '../models/term';
import { catchError, map } from 'rxjs/operators';
import { BaseService } from 'app/shared/services/base.service';
import { HttpCacheManager, withCache } from '@ngneat/cashew';
import { BackendError } from 'app/shared/models/backend-error';
import { CreateTermDto } from '../dto/create-term.dto';

@Injectable({
    providedIn: 'root'
})
export class TermService extends BaseService {
    constructor(private http: HttpClient, private cacheManager: HttpCacheManager) {
        super();
    }

    getById(id: number): Observable<Term> {
        return this.http.get<Term>(`${environment.backendUrl}/term/${id}`, {
            context: withCache()
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(term => new Term(term))
        )
    }

    getAll(): Observable<Term[]> {
        return this.http.get<Term[]>(`${environment.backendUrl}/term`, {
            context: withCache()
        })
        .pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(terms => terms.map(term => new Term(term)))
        );
    }

    create(dto: CreateTermDto): Observable<Term> {
        return this.http.post<Term>(`${environment.backendUrl}/term`, dto).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(pool => {
                this.cacheManager.clear();
                return new Term(pool);
            })
        );
    }

    update(termId: number, dto: CreateTermDto): Observable<Term> {
        return this.http.put<Term>(`${environment.backendUrl}/term/${termId}`, dto).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(term => {
                this.cacheManager.clear();
                return new Term(term);
            })
        );
    }
}
