import { Component, Input, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Course } from 'app/features/courses/models/course';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { BackendError } from 'app/shared/models/backend-error';
import { takeUntil } from 'rxjs/operators';
import { TaskPool } from '../../models/task-pool';
import { TaskPoolService } from '../../services/task-pool.service';

@Component({
  selector: 'ngx-task-pool-courses',
  templateUrl: './task-pool-courses.component.html',
  styleUrls: ['./task-pool-courses.component.scss']
})
export class TaskPoolCoursesComponent extends BaseComponent implements OnInit {
  @Input() pool: TaskPool;
  courses: Course[] = [];

  loading: boolean = true;
  error: BackendError;

  constructor(
    private taskPoolService: TaskPoolService,
    private router: Router,
    private route: ActivatedRoute) {
    super();
  }

  ngOnInit(): void {
    this.taskPoolService.getCourses(this.pool.id)
    .pipe(
      takeUntil(this.unsubscribe)
    )
    .subscribe(courses => {
      this.courses = courses;
      this.loading = false;
    }, error => {
      if (BackendError.isBackendError(error)) {
        this.error = error;
        this.loading = false;
      } else {
        throw error;
      }
    });
  }
}
