import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { takeUntil } from 'rxjs/operators';
import { FileTree } from 'app/shared/models/file-tree';
import { AppStateService } from 'app/core/services/app.state.service';

export interface File {
  reference: FileTree;
  content?: string | Blob;
}

@Component({
  selector: 'ngx-source-code-viewer',
  templateUrl: './source-code-viewer.component.html',
  styleUrls: ['./source-code-viewer.component.scss']
})
export class SourceCodeViewerComponent extends BaseComponent implements OnInit {
  @Input() file: File;
  @Output() downloadTreeEvent = new EventEmitter<FileTree>();
  @Output() toggleEvent = new EventEmitter<void>();
  darkMode = false;

  constructor(
    private appStateService: AppStateService,
    ) {
    super();
  }

  ngOnInit(): void {
    this.appStateService.darkMode$
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(darkMode => this.darkMode = darkMode);
  }

  open() {
    this.downloadTreeEvent.emit(this.file.reference);
  }

  toggle() {
    this.toggleEvent.emit();
  }
}
