import { HttpParams } from "@angular/common/http";
import { Page } from "./page";

export interface DataQueryParams {
    sortOrder?: string;
    sortBy?: string;
    pageNumber?: number;
    pageSize?: number;
}

export namespace DataQueryParams {
    export function next<T>(page?: Page<T>, old?: DataQueryParams): DataQueryParams {
        if(page == null) {
            return null;
        }

        let params: DataQueryParams = { 
            pageNumber: page.number + 1
        };

        if(old) {
            params.pageSize = old.pageSize;
            params.sortBy = old.sortBy;
            params.sortOrder = old.sortOrder;
        }

        return params;
    }

    export function toHttpParams(params?: DataQueryParams): HttpParams {
        let httpParams = new HttpParams()
    
        if(params?.sortBy && params?.sortOrder) {
            httpParams = httpParams.set('sort', `${params.sortBy},${params.sortOrder}`);
        }

        if(params?.pageNumber != null) {
            let pageNumber = params.pageNumber;
            httpParams = httpParams.set('page', pageNumber);
        }

        if(params?.pageSize != null) {
            let pageSize = params.pageSize;
            httpParams = httpParams.set('size', pageSize);
        }

        return httpParams;
    }
}