import { NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { LoadingScreenComponent } from './components/loading-screen/loading-screen.component';
import { NotificationService } from './services/notification.service';
import { HttpClientModule, HTTP_INTERCEPTORS } from '@angular/common/http';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { NbMomentDateModule } from '@nebular/moment';
import { NbAlertModule, NbCardModule, NbIconModule, NbInputModule, NbBadgeModule, NbListModule, NbButtonModule, NbAccordionModule, NbDatepickerModule, NbTimepickerModule, NbToggleModule, NbSpinnerModule, NbCheckboxModule, NbSelectModule, NbTreeGridModule, NbFormFieldModule, NbActionsModule, NbContextMenuModule, NbLayoutModule, NbMenuModule, NbSearchModule, NbSidebarModule, NbUserModule, NbAutocompleteComponent, NbAutocompleteModule, NbTabsetModule, NbProgressBarModule, NbTagModule, NbPopoverModule, NbStepperModule } from '@nebular/theme';
import { BaseComponent } from './components/base/base.component';
import { FileBrowserComponent } from './components/file-browser/file-browser.component';
import { SourceCodeViewerComponent } from './components/source-code-viewer/source-code-viewer.component';
import { ValidationHintComponent } from './components/validation-hint/validation-hint.component';
import { ValidationSummaryComponent } from './components/validation-summary/validation-summary.component';
import { NbEvaIconsModule } from '@nebular/eva-icons';
import { NbSecurityModule } from '@nebular/security';
import { NgxDropzoneModule } from 'ngx-dropzone';
import { DropzoneComponent } from './components/dropzone/dropzone.component';
import { DragDropModule } from '@angular/cdk/drag-drop';
import { EditorModule } from '@tinymce/tinymce-angular';
import { NgxSliderModule } from '@angular-slider/ngx-slider';
import { MathjaxModule } from 'app/lib/mathjax/mathjax.module';
import { CodemirrorModule } from '@ctrl/ngx-codemirror';
import { RouteReuseLifeCycleDirective } from './route-reuse-life.cycle.directive';
import { NgJoinPipeModule } from 'angular-pipes';
import { NgPluckPipeModule } from 'angular-pipes';
import { NgWherePipeModule } from 'angular-pipes';
import { NgRoundPipeModule } from 'angular-pipes';
import { NgMapPipeModule } from 'angular-pipes';
import { PropPipe } from './pipes/prop.pipe';
import { FnPipe } from './pipes/fn.pipe';
import { ErrorAlertComponent } from './components/error-alert/error-alert.component';
import { RouterModule } from '@angular/router';
import { DialogConfirmationComponent } from './dialogs/dialog-confirmation/dialog-confirmation.component';
import { UserPickerComponent } from './components/user-picker/user-picker.component';
import { ErrorContainerComponent } from './components/error-container/error-container.component';
import { PageContainerComponent } from './components/page-container/page-container.component';
import { BreadcrumbsComponent } from './components/breadcrumbs/breadcrumbs.component';
import { CodemirrorThemePipe } from './pipes/codemirror-theme.pipe';
import { CodemirrorModePipe } from './pipes/codemirror-mode.pipe';
import { CodemirrorHighlightPipe } from './pipes/codemirror-highlight';
import { UserNamePipe } from './pipes/user-name.pipe';
import { NavigationListComponent } from './components/navigation-list/navigation-list.component';
import { NavigationListItemComponent } from './components/navigation-list-item/navigation-list-item.component';
import { TrustUrlPipe } from './pipes/trust-url.pipe';

@NgModule({
  declarations: [
    LoadingScreenComponent,
    BaseComponent,
    FileBrowserComponent,
    SourceCodeViewerComponent,
    ValidationHintComponent,
    ValidationSummaryComponent,

    PropPipe,
    FnPipe,
    CodemirrorThemePipe,
    CodemirrorModePipe,
    CodemirrorHighlightPipe,
    UserNamePipe,
    TrustUrlPipe,
    
    RouteReuseLifeCycleDirective,

    DialogConfirmationComponent,
    DropzoneComponent,
    ErrorAlertComponent,
    UserPickerComponent,
    ErrorContainerComponent,
    PageContainerComponent,
    BreadcrumbsComponent,
    NavigationListComponent,
    NavigationListItemComponent,
  ],
  exports: [
    HttpClientModule,
    NbAlertModule,
    NbCardModule,
    NbIconModule,
    NbInputModule,
    NbBadgeModule,
    NbListModule,
    NbButtonModule,
    NbAccordionModule,
    FormsModule,
    ReactiveFormsModule,
    NbDatepickerModule,
    NbTimepickerModule,
    NbMomentDateModule,
    NbSelectModule,
    NbTabsetModule,
    NbToggleModule,
    NbCheckboxModule,
    NbSpinnerModule,
    NbFormFieldModule,
    NbEvaIconsModule,
    NbSecurityModule,
    NbContextMenuModule,
    NbSidebarModule,
    NbSearchModule,
    NbActionsModule,
    NbUserModule,
    NbMenuModule,
    NbLayoutModule,
    NgxSliderModule,
    NbProgressBarModule,
    NbStepperModule,
    NbAutocompleteModule,
    NbTagModule,
    NbPopoverModule,
    DragDropModule,
    CodemirrorModule,
    NgJoinPipeModule,
    NgPluckPipeModule,
    NgWherePipeModule,
    NgRoundPipeModule,
    NgMapPipeModule,

    PropPipe,
    FnPipe,
    CodemirrorThemePipe,
    CodemirrorModePipe,
    CodemirrorHighlightPipe,
    UserNamePipe,
    TrustUrlPipe,
    
    RouteReuseLifeCycleDirective,
    LoadingScreenComponent,
    BaseComponent,
    FileBrowserComponent,
    SourceCodeViewerComponent,
    ValidationHintComponent,
    ValidationSummaryComponent,
    ErrorAlertComponent,
    ErrorContainerComponent,
    PageContainerComponent,
    BreadcrumbsComponent,
    NavigationListComponent,
    NavigationListItemComponent,
    
    DialogConfirmationComponent,
    NgxDropzoneModule,
    DropzoneComponent,
    UserPickerComponent,
    EditorModule,
    MathjaxModule
  ],
  imports: [
    CommonModule,
    HttpClientModule,
    RouterModule,
    NbAlertModule,
    NbEvaIconsModule,
    NbStepperModule,
    NbPopoverModule,
    NbSecurityModule,
    NbContextMenuModule,
    NbSidebarModule,
    NbSearchModule,
    NbTabsetModule,
    NbActionsModule,
    NbUserModule,
    NbMenuModule,
    NgxSliderModule,
    NbLayoutModule,
    NbCardModule,
    NbIconModule,
    NbInputModule,
    NbBadgeModule,
    NbListModule,
    NbButtonModule,
    NbTagModule,
    NbAccordionModule,
    NbProgressBarModule,
    FormsModule,
    ReactiveFormsModule,
    NbDatepickerModule,
    NbTimepickerModule,
    NbMomentDateModule,
    NbToggleModule,
    NbSpinnerModule,
    NbTreeGridModule,
    NbFormFieldModule,
    NbAutocompleteModule,
    NgxDropzoneModule,
    DragDropModule,
    EditorModule,
    MathjaxModule.forChild(),
    CodemirrorModule,
    NgJoinPipeModule,
    NgPluckPipeModule,
    NgWherePipeModule,
    NgRoundPipeModule,
    NgMapPipeModule
  ]
})
export class SharedModule { }
