package de.hsrm.sls.subato.aor.core.imp;

import de.hsrm.sls.subato.aor.core.model.ModuleNode;
import de.hsrm.sls.subato.modules.progmodule.ProgramModule;
import de.hsrm.sls.subato.modules.progmodule.ProgramModuleRepository;
import de.hsrm.sls.subato.modules.program.DegreeProgramRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

import static de.hsrm.sls.subato.aor.core.imp.ModuleFrequencyConverter.convertFrequency;

@Service
@Transactional
public class ProgramModuleImportService {
    private static Logger logger = LoggerFactory.getLogger(ProgramModuleImportService.class);

    @Autowired
    DegreeProgramRepository programRepository;

    @Autowired
    ProgramModuleRepository programModuleRepository;

    public void importModules(List<ModuleNode> moduleNodes) {
        var existingPrograms = programRepository.findAll();

        for(var moduleNode : moduleNodes) {
            var programModule = programModuleRepository.findByAorId(moduleNode.getId());
            if(programModule == null) {
                var program = existingPrograms.stream().filter(p -> p.getAorId().equals(moduleNode.getProgramId()))
                        .findFirst()
                        .orElse(null);

                if(program == null) {
                    logger.error("moduleNode %s: program %s not found".formatted(moduleNode.getId(), moduleNode.getProgramId()));
                    continue;
                }

                programModule = new ProgramModule();
                programModule.setName(moduleNode.getName());
                programModule.setFrequency(convertFrequency(moduleNode.getFrequency()));
                programModule.setShortName(moduleNode.getMatchcode());
                programModule.setScheduledTerm(moduleNode.getDueInTerm());
                programModule.setProgram(program);
                programModule.setAorId(moduleNode.getId());
                programModuleRepository.save(programModule);
            }
        }
    }
}
