package de.hsrm.sls.subato.auth.masking;

import de.hsrm.sls.subato.auth.Role;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.tasks.task.TaskDetailDto;
import de.hsrm.sls.subato.tasks.task.TaskSummaryDto;
import org.springframework.stereotype.Component;

@Component
public class TaskMasker implements Masker {
    @Override
    public void mask(Object obj, SubatoPrincipal principal) {
        if(obj instanceof TaskDetailDto) {
            mask((TaskDetailDto) obj, principal);
        }

        if(obj instanceof TaskSummaryDto) {
            mask((TaskSummaryDto) obj, principal);
        }
    }

    private void mask(TaskDetailDto obj, SubatoPrincipal principal) {
        if(principal == null || !principal.hasAnyRole(Role.Staff, Role.SuperAdmin)) {
            obj.setEvaluator(null);
            obj.setPool(null);
            obj.setFeedbackStats(null);
        }
    }

    private void mask(TaskSummaryDto obj, SubatoPrincipal principal) {
        if(principal == null || !principal.hasAnyRole(Role.Staff, Role.SuperAdmin)) {
            obj.setPool(null);
            obj.setFeedbackStats(null);
        }
    }

    @Override
    public boolean supports(Class clazz) {
        return clazz.equals(TaskDetailDto.class) || clazz.equals(TaskSummaryDto.class);
    }
}
