package de.hsrm.sls.subato.auth.policy.solutions;

import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyRule;
import de.hsrm.sls.subato.auth.policy.exercises.ExercisesAction;
import de.hsrm.sls.subato.courses.course.SubmitPolicy;
import de.hsrm.sls.subato.exercises.exercise.Exercise;
import de.hsrm.sls.subato.tasks.task.Task;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.stereotype.Component;

/**
 * The current user is allowed to submit a solution to an exercise within a certain course
 */
@Component
public class SolutionSubmitExercisePolicy implements PolicyRule {
    @Override
    public boolean check(CheckPolicyRequest request) {
        if(!request.matches(SolutionsAction.SOLUTION_SUBMIT_EXERCISE)) {
            return false;
        }

        var taskInstance = (Pair<Exercise, Task>) request.getObj();
        var course = taskInstance.getKey().getCourse();
        return course.getSubmitPolicy() == SubmitPolicy.EVERYONE ||
                (request.authenticated() && request.getSubject().isParticipant(course.getId()));
    }
}
