package de.hsrm.sls.subato.courses.event;

import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.auth.policy.courses.CoursesAction;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.server.ResponseStatusException;

import static org.springframework.http.HttpStatus.NOT_FOUND;

@RestController
@Tag(name = "Kursverwaltung")
public class CourseUnitEventController {

    @Autowired
    private CourseUnitEventService courseUnitEventService;

    @Autowired
    private CourseUnitEventMapper courseUnitEventMapper;

    @Autowired
    private PolicyEvaluator policy;

    @PutMapping("/course_unit_event/{id}/cancellation")
    public CourseUnitEventSummaryDto updateCancellation(@PathVariable Long id, @RequestBody UpdateEventCancellationDto dto) {
        try {
            var event = courseUnitEventService.findById(id);
            policy.checkAndRaise(CheckPolicyRequest.fromPrincipal(event.getUnit(), CoursesAction.COURSE_UNIT_UPDATE));

            event = courseUnitEventService.updateCancellation(event, dto);
            return courseUnitEventMapper.mapSummary(event);
        } catch(CourseUnitEventNotFoundException ex) {
            throw new ResponseStatusException(NOT_FOUND, CourseUnitEventNotFoundException.REASON);
        }
    }
}
