package de.hsrm.sls.subato.courses.file;

import de.hsrm.sls.subato.courses.course.Course;
import de.hsrm.sls.subato.shared.fs.FileContent;
import de.hsrm.sls.subato.shared.helper.FileHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;

@Service
public class CourseFileService {
    @Autowired
    private CourseFileRepository courseFileRepository;

    @Autowired
    private CourseFileContentRepository courseFileContentRepository;

    public CourseFile findById(long id) {
        return findById(id, true);
    }

    public CourseFile findById(long id, boolean raiseException) {
        var courseFile = courseFileRepository.findById(id);

        if(raiseException) {
            return courseFile.orElseThrow(() -> new CourseFileNotFoundException(id));
        } else {
            return courseFile.orElse(null);
        }
    }

    public File getFile(CourseFile courseFile) {
        return courseFileContentRepository.getFile(courseFile);
    }

    public CourseFile create(FileContent content, Course course, CreateCourseFileDto metadata) {

        var courseFile = new CourseFile();
        courseFile.setDescription(metadata.getDescription());
        courseFile.setCourse(course);
        courseFile.setName(content.getFileName());

        var contentType = FileHelper.getContentType(content.getFileName());
        var isSourceFile = FileHelper.isSourceFile(contentType);
        courseFile.setSrc(isSourceFile);

        courseFile = courseFileRepository.save(courseFile);
        courseFileContentRepository.store(content, courseFile);

        return courseFile;
    }

    public void delete(CourseFile courseFile) {
        courseFileRepository.delete(courseFile);
        courseFileContentRepository.delete(courseFile);
    }
}
