package de.hsrm.sls.subato.modules.term;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import java.util.List;

@Service
public class TermService {
    @Autowired
    private TermRepository termRepository;

    public Term findById(long id) {
        return termRepository.findById(id).orElseThrow(() -> new TermNotFoundException(id));
    }

    public List<Term> findAll() {
        return termRepository.findAll(Sort.by(Sort.Direction.DESC, "lecturesStartAt"));
    }

    private void updateTerm(Term term, CreateTermDto dto) {
        term.setName(dto.getName());
        term.setLecturesStartAt(dto.getLecturesStartAt());
        term.setLecturesEndAt(dto.getLecturesEndAt());
    }

    /**
     * @throws ConstraintViolationException if invalid
     */
    private void validateTerm(Term term) {
        var validator = Validation.buildDefaultValidatorFactory().getValidator();
        var violations = validator.validate(term);
        if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
        }
    }

    /**
     * @param dto attributes for the new term
     * @return created term
     * @throws ConstraintViolationException
     */
    public Term create(CreateTermDto dto) {
        var term = new Term();
        updateTerm(term, dto);
        validateTerm(term);

        return termRepository.save(term);
    }

    /**
     * @throws ConstraintViolationException
     * @throws TermNotFoundException
     */
    public Term update(long id, CreateTermDto dto) {
        var term = findById(id);
        updateTerm(term, dto);
        validateTerm(term);

        return termRepository.save(term);
    }
}
