package de.hsrm.sls.subato.shared.fs;

import com.fasterxml.jackson.annotation.JsonIgnore;
import de.hsrm.sls.subato.shared.helper.FileHelper;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;
import java.util.function.Predicate;

public class FileTree {
    private UUID uuid;

    private boolean isSrc;
    private String name;

    /**
     * relative to the base path, where this file tree was created from
     */
    private String path;
    private String contentType;

    private boolean isDirectory;

    private List<FileTree> children = new ArrayList<>();

    public FileTree() {}

    public FileTree(String name, String path, String contentType, boolean isSrc) {
        this.name = name;
        this.path = path;
        this.contentType = contentType;
        this.uuid = UUID.randomUUID();
        this.isSrc = isSrc;
        this.children = new ArrayList<>();
    }

    public boolean isSrc() {
        return isSrc;
    }

    public void setSrc(boolean src) {
        isSrc = src;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public UUID getUuid() {
        return uuid;
    }

    public void setUuid(UUID uuid) {
        this.uuid = uuid;
    }

    public String getPath() {
        return path;
    }

    public void setPath(String path) {
        this.path = path;
    }

    public List<FileTree> getChildren() {
        return children;
    }

    public void setChildren(List<FileTree> children) {
        this.children = children;
    }

    public boolean isDirectory() {
        return isDirectory;
    }

    public void setDirectory(boolean directory) {
        isDirectory = directory;
    }

    public String getContentType() {
        return contentType;
    }

    public void setContentType(String contentType) {
        this.contentType = contentType;
    }

    @JsonIgnore
    public String getAbsolutePath(Path basePath) {
        return basePath.resolve(path).toString();
    }

    @JsonIgnore
    public File getFile(Path basePath) {
        return Path.of(getAbsolutePath(basePath)).toFile();
    }

    /*
    @JsonIgnore
    public File getFile() {
        return Path.of(getAbsolutePath(null)).toFile();
    }

    @JsonIgnore
    public String content() {
        try {
            return Files.readString(Path.of(getAbsolutePath(null)), StandardCharsets.UTF_8);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @JsonIgnore
    public byte[] data(Path basePath) {
        try {
            return Files.readAllBytes(Path.of(getAbsolutePath(basePath)));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @JsonIgnore
    public byte[] data() {
        return data(null);
    }

     */

    public FileTree find(Predicate<FileTree> pred) {
        for(var item : children) {
            if(pred.test(item)) {
                return item;
            }

            var match = item.find(pred);
            if(match != null) {
                return match;
            }
        }

        return null;
    }

    public FileTree find(String path) {
        return find(i -> i.getPath().equals(path));
    }

    public static FileTree from(File file, String basePath, boolean recursive, boolean includeRoot) {
        if(!file.exists()) {
            return null;
        }

        var contentType = FileHelper.getContentType(file);
        var isSourceFile = FileHelper.isSourceFile(contentType);

        var fileName = includeRoot ? file.getName() : "";
        var item = new FileTree(fileName, Path.of(basePath).resolve(fileName).toString(), contentType, isSourceFile);

        if(file.isDirectory()) {
            item.setDirectory(true);

            if(recursive) {
                var relativePath = Path.of(basePath).resolve(Path.of(fileName));

                for (var f : file.listFiles()) {
                    item.getChildren().add(from(f, relativePath.toString(), recursive, true));
                }
            }
        }

        return item;
    }
}
