package de.hsrm.sls.subato.shared.fs;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;

@Service
public class FileTreeCache {
    private FileTree buildTree(Path path) {
        var file = path.toFile();

        if(!file.exists()) {
            throw new RuntimeException("file/directory does not exist: " + path);
        }

        if(!file.isDirectory()) {
            throw new RuntimeException("path is not a directory");
        }

        return FileTree.from(file, "", true, false);
    }

    public synchronized void saveTree(FileTree tree, Path basePath) {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            var serialized = objectMapper.writeValueAsString(tree);
            var treeFile = basePath.resolve("tree.json").toFile();
            Files.write(treeFile.toPath(), serialized.getBytes(StandardCharsets.UTF_8));
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public synchronized FileTree getTree(Path basePath) {
        var cacheFilePath = basePath.resolve("tree.json");
        if(!cacheFilePath.toFile().exists()) {
            var tree = buildTree(basePath);
            saveTree(tree, basePath);
            return tree;
        }

        var objectMapper = new ObjectMapper();
        try {
            var treeContent = Files.readString(cacheFilePath, StandardCharsets.UTF_8);
            var tree = objectMapper.readValue(treeContent, FileTree.class);
            return tree;
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
