package de.hsrm.sls.subato.shared.helper;

import org.springframework.core.io.Resource;
import org.springframework.core.io.UrlResource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.nio.file.Files;
import java.nio.file.Path;

public class ResourceHelper {
    public static Resource load(File file) {
        try {
            Resource resource = new UrlResource(file.toPath().toUri());

            if (resource.exists() || resource.isReadable()) {
                return resource;
            } else {
                throw new RuntimeException("Could not read the file!");
            }
        } catch (MalformedURLException e) {
            throw new RuntimeException("Error: " + e.getMessage());
        }
    }

    public static ResponseEntity buildFileResponse(Resource resource, boolean asAttachment) throws IOException {
        return buildFileResponse(resource, resource.getFilename(), asAttachment);
    }

    public static ResponseEntity buildFileResponse(Resource resource, String fileName, boolean asAttachment) throws IOException {
        String contentType = FileHelper.getContentType(fileName);
        var disposition = asAttachment ? "attachment" : "inline";

        return ResponseEntity.ok().contentType(MediaType.parseMediaType(contentType))
                .header(HttpHeaders.CONTENT_DISPOSITION, disposition + "; filename=\"" + fileName + "\"")
                .body(resource);
    }
}
