package de.hsrm.sls.subato.tasks.stef.parser.meta;

import de.hsrm.sls.subato.tasks.stef.Meta;
import de.hsrm.sls.subato.tasks.stef.SubmissionModes;
import de.hsrm.sls.subato.tasks.stef.file.FileType;
import de.hsrm.sls.subato.tasks.stef.parser.InvalidSpecException;
import de.hsrm.sls.subato.tasks.stef.parser.meta.xml.SubmissionModeXml;
import de.hsrm.sls.subato.tasks.stef.parser.meta.xml.SubmissionModesXml;
import de.hsrm.sls.subato.tasks.stef.parser.meta.xml.TaskXml;
import de.hsrm.sls.subato.tasks.task.SubmissionMode;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class SubmissionModeExtractor {
    public void extract(TaskXml doc, Meta meta) {
        if(doc.getSubmissionModes() != null) {
            if(doc.getSubmissionModes().getSubmissionModes().size() == 0) {
                throw new InvalidSpecException("Wenn %s konfiguriert wird, muss mind. ein %s konfiguriert werden".formatted(TaskXml.SUBMISSION_MODES, SubmissionModesXml.SUBMISSION_MODE));
            }

            var submissionModes = new ArrayList<SubmissionMode>();
            for (var submissionMode : doc.getSubmissionModes().getSubmissionModes()) {
                if(submissionMode.getName() == null) {
                    throw new InvalidSpecException("%s-Attribut in %s fehlt".formatted(SubmissionModeXml.NAME, SubmissionModesXml.SUBMISSION_MODE));
                }

                try {
                    submissionModes.add(SubmissionMode.valueOf(submissionMode.getName()));
                } catch (IllegalArgumentException ex) {
                    throw new InvalidSpecException("SubmissionMode %s existiert nicht im System".formatted(submissionMode.getName()));
                }
            }

            SubmissionMode defaultMode = null;
            if(doc.getSubmissionModes().getDefaultMode() != null) {
                try {
                     defaultMode = SubmissionMode.valueOf(doc.getSubmissionModes().getDefaultMode());

                    if (!submissionModes.contains(defaultMode)) {
                        throw new InvalidSpecException("SubmissionMode %s kann nicht als default verwendet werden, da er nicht konfiguriert wurde".formatted(doc.getSubmissionModes().getDefaultMode()));
                    }
                } catch(IllegalArgumentException ex) {
                    throw new InvalidSpecException("SubmissionMode %s existiert nicht im System".formatted(doc.getSubmissionModes().getDefaultMode()));
                }
            }

            meta.setSubmissionModes(new SubmissionModes(submissionModes, defaultMode != null ? defaultMode : getDefaultMode(submissionModes)));
        } else {
            var modes = new ArrayList<SubmissionMode>();
            modes.add(SubmissionMode.FILE);
            if(meta.getFiles().stream().filter(f -> f.getType() == FileType.TEMPLATE).count() <= 1) {
                modes.add(SubmissionMode.TEXT);
            }

            var submissionModes = new SubmissionModes(modes, getDefaultMode(modes));
            meta.setSubmissionModes(submissionModes);
        }

    }

    private SubmissionMode getDefaultMode(List<SubmissionMode> modes) {
        return modes.contains(SubmissionMode.TEXT) ? SubmissionMode.TEXT : SubmissionMode.FILE;
    }
}
